@@ -4,8 +4,12 @@
 
 # thanks Moof (aka Giles Antonio Radford) for some of these
 
-import os, sys
-from unittest import TestCase
+import errno
+import os
+import socket
+import sys
+import tempfile
+import urllib
 
 import mechanize
 from mechanize import build_opener, install_opener, urlopen, urlretrieve
@@ -14,13 +18,10 @@ from mechanize import CookieJar, HTTPCookieProcessor, \
      HTTPEquivProcessor, HTTPRedirectHandler, \
      HTTPRedirectDebugProcessor, HTTPResponseDebugProcessor
 from mechanize._rfc3986 import urljoin
-
-# XXX
-# document twisted.web2 install (I forgot how I did it -- reinstall!)
-# implement remaining stuff used by functional_tests.py
-# in twisted-localserver.py:
-#   - 302 followed by 404 response
-#   - helper cgi script for cookies &c.
+from mechanize._util import hide_experimental_warnings, \
+    reset_experimental_warnings
+import mechanize._sockettimeout
+from mechanize._testcase import TestCase
 
 #from cookielib import CookieJar
 #from urllib2 import build_opener, install_opener, urlopen
@@ -43,10 +44,76 @@ def sanepathname2url(path):
     # XXX don't ask me about the mac...
     return urlpath
 
-class SimpleTests(TestCase):
+
+def read_file(filename):
+    fh = open(filename)
+    try:
+        return fh.read()
+    finally:
+        fh.close()
+
+
+class SocketTimeoutTest(TestCase):
+
+    # the timeout tests in this module aren't full functional tests: in order
+    # to speed things up, don't actually call .settimeout on the socket.  XXX
+    # allow running the tests against a slow server with a real timeout
+
+    def _monkey_patch_socket(self):
+        class Delegator(object):
+            def __init__(self, delegate):
+                self._delegate = delegate
+            def __getattr__(self, name):
+                return getattr(self._delegate, name)
+
+        assertEquals = self.assertEquals
+
+        class TimeoutLog(object):
+            AnyValue = object()
+            def __init__(self):
+                self._nr_sockets = 0
+                self._timeouts = []
+                self.start()
+            def start(self):
+                self._monitoring = True
+            def stop(self):
+                self._monitoring = False
+            def socket_created(self):
+                if self._monitoring:
+                    self._nr_sockets += 1
+            def settimeout_called(self, timeout):
+                if self._monitoring:
+                    self._timeouts.append(timeout)
+            def verify(self, value=AnyValue):
+                if sys.version_info[:2] < (2, 6):
+                    # per-connection timeout not supported in Python 2.5
+                    self.verify_default()
+                else:
+                    assertEquals(len(self._timeouts), self._nr_sockets)
+                    if value is not self.AnyValue:
+                        for timeout in self._timeouts:
+                            assertEquals(timeout, value)
+            def verify_default(self):
+                assertEquals(len(self._timeouts), 0)
+
+        log = TimeoutLog()
+        def settimeout(timeout):
+            log.settimeout_called(timeout)
+        orig_socket = socket.socket
+        def make_socket(*args, **kwds):
+            sock = Delegator(orig_socket(*args, **kwds))
+            log.socket_created()
+            sock.settimeout = settimeout
+            return sock
+        self.monkey_patch(socket, "socket", make_socket)
+        return log
+
+
+class SimpleTests(SocketTimeoutTest):
     # thanks Moof (aka Giles Antonio Radford)
 
     def setUp(self):
+        super(SimpleTests, self).setUp()
         self.browser = mechanize.Browser()
 
     def test_simple(self):
@@ -56,6 +123,46 @@ class SimpleTests(TestCase):
         self.browser.open('/mechanize/')
         self.assertEqual(self.browser.title(), 'mechanize')
 
+    def test_basic_auth(self):
+        uri = urljoin(self.uri, "basic_auth")
+        self.assertRaises(mechanize.URLError, self.browser.open, uri)
+        self.browser.add_password(uri, "john", "john")
+        self.browser.open(uri)
+        self.assertEqual(self.browser.title(), 'Basic Auth Protected Area')
+
+    def test_digest_auth(self):
+        uri = urljoin(self.uri, "digest_auth")
+        self.assertRaises(mechanize.URLError, self.browser.open, uri)
+        self.browser.add_password(uri, "digestuser", "digestuser")
+        self.browser.open(uri)
+        self.assertEqual(self.browser.title(), 'Digest Auth Protected Area')
+
+    def test_open_with_default_timeout(self):
+        timeout_log = self._monkey_patch_socket()
+        self.browser.open(self.uri)
+        self.assertEqual(self.browser.title(), 'Python bits')
+        timeout_log.verify_default()
+
+    def test_open_with_timeout(self):
+        timeout_log = self._monkey_patch_socket()
+        timeout = 10.
+        self.browser.open(self.uri, timeout=timeout)
+        self.assertEqual(self.browser.title(), 'Python bits')
+        timeout_log.verify(timeout)
+
+    def test_urlopen_with_default_timeout(self):
+        timeout_log = self._monkey_patch_socket()
+        response = mechanize.urlopen(self.uri)
+        self.assert_contains(response.read(), "Python bits")
+        timeout_log.verify_default()
+
+    def test_urlopen_with_timeout(self):
+        timeout_log = self._monkey_patch_socket()
+        timeout = 10.
+        response = mechanize.urlopen(self.uri, timeout=timeout)
+        self.assert_contains(response.read(), "Python bits")
+        timeout_log.verify(timeout)
+
     def test_302_and_404(self):
         # the combination of 302 and 404 (/redirected is configured to redirect
         # to a non-existent URL /nonexistent) has caused problems in the past
@@ -79,7 +186,7 @@ class SimpleTests(TestCase):
         self.assertEqual(self.browser.response().read(), data)
 
     def test_error_recovery(self):
-        self.assertRaises(OSError, self.browser.open,
+        self.assertRaises(mechanize.URLError, self.browser.open,
                           'file:///c|thisnoexistyiufheiurgbueirgbue')
         self.browser.open(self.uri)
         self.assertEqual(self.browser.title(), 'Python bits')
@@ -90,20 +197,55 @@ class SimpleTests(TestCase):
         self.assertEqual(r.code, 200)
         self.assert_("GeneralFAQ.html" in r.read(2048))
 
+    def test_refresh(self):
+        def refresh_request(seconds):
+            uri = urljoin(self.uri, "/cgi-bin/cookietest.cgi")
+            val = urllib.quote_plus('%d; url="%s"' % (seconds, self.uri))
+            return uri + ("?refresh=%s" % val)
+        self.browser.set_handle_refresh(True, honor_time=False)
+        r = self.browser.open(refresh_request(5))
+        self.assertEqual(r.geturl(), self.uri)
+        # Set a maximum refresh time of 30 seconds (these long refreshes tend
+        # to be there only because the website owner wants you to see the
+        # latest news, or whatever -- they're not essential to the operation of
+        # the site, and not really useful or appropriate when scraping).
+        refresh_uri = refresh_request(60)
+        self.browser.set_handle_refresh(True, max_time=30., honor_time=True)
+        r = self.browser.open(refresh_uri)
+        self.assertEqual(r.geturl(), refresh_uri)
+        # allow long refreshes (but don't actually wait 60 seconds)
+        self.browser.set_handle_refresh(True, max_time=None, honor_time=False)
+        r = self.browser.open(refresh_request(60))
+        self.assertEqual(r.geturl(), self.uri)
+
     def test_file_url(self):
         url = "file://%s" % sanepathname2url(
             os.path.abspath('functional_tests.py'))
         r = self.browser.open(url)
         self.assert_("this string appears in this file ;-)" in r.read())
 
+    def test_open_local_file(self):
+        # Since the file: URL scheme is not well standardised, Browser has a
+        # special method to open files by name, for convenience:
+        br = mechanize.Browser()
+        response = br.open_local_file("mechanize/_mechanize.py")
+        self.assert_("def open_local_file(self, filename):" in
+                     response.get_data())
+
     def test_open_novisit(self):
         def test_state(br):
             self.assert_(br.request is None)
             self.assert_(br.response() is None)
             self.assertRaises(mechanize.BrowserStateError, br.back)
         test_state(self.browser)
+        uri = urljoin(self.uri, "bits")
         # note this involves a redirect, which should itself be non-visiting
-        r = self.browser.open_novisit(urljoin(self.uri, "bits"))
+        r = self.browser.open_novisit(uri)
+        test_state(self.browser)
+        self.assert_("GeneralFAQ.html" in r.read(2048))
+
+        # Request argument instead of URL
+        r = self.browser.open_novisit(mechanize.Request(uri))
         test_state(self.browser)
         self.assert_("GeneralFAQ.html" in r.read(2048))
 
@@ -138,6 +280,14 @@ class ResponseTests(TestCase):
                          r.get_data(),
                          "Hello ClientCookie functional test suite.\n")
 
+    def test_seek_wrapper_class_name(self):
+        opener = mechanize.UserAgent()
+        opener.set_seekable_responses(True)
+        try:
+            opener.open(urljoin(self.uri, "nonexistent"))
+        except mechanize.HTTPError, exc:
+            self.assert_("HTTPError instance" in repr(exc))
+
     def test_no_seek(self):
         # should be possible to turn off UserAgent's .seek() functionality
         def check_no_seek(opener):
@@ -252,13 +402,13 @@ class ResponseTests(TestCase):
                          "Hello ClientCookie functional test suite.\n")
 
 
-class FunctionalTests(TestCase):
+class FunctionalTests(SocketTimeoutTest):
 
     def test_referer(self):
         br = mechanize.Browser()
+        br.set_handle_refresh(True, honor_time=False)
         referer = urljoin(self.uri, "bits/referertest.html")
         info = urljoin(self.uri, "/cgi-bin/cookietest.cgi")
-
         r = br.open(info)
         self.assert_(referer not in r.get_data())
 
@@ -315,37 +465,47 @@ class FunctionalTests(TestCase):
                 mechanize.RobotExclusionError,
                 opener.open, urljoin(self.uri, "norobots"))
 
-    def test_urlretrieve(self):
-        url = urljoin(self.uri, "/mechanize/")
-        test_filename = "python.html"
-        def check_retrieve(opener, filename, headers):
-            self.assertEqual(headers.get('Content-Type'), 'text/html')
-            f = open(filename)
-            data = f.read()
-            f.close()
-            opener.close()
-            from urllib import urlopen
-            r = urlopen(url)
-            self.assertEqual(data, r.read())
-            r.close()
+    def _check_retrieve(self, url, filename, headers):
+        from urllib import urlopen
+        self.assertEqual(headers.get('Content-Type'), 'text/html')
+        self.assertEqual(read_file(filename), urlopen(url).read())
 
+    def test_retrieve_to_named_file(self):
+        url = urljoin(self.uri, "/mechanize/")
+        test_filename = os.path.join(self.make_temp_dir(), "python.html")
         opener = mechanize.build_opener()
         verif = CallbackVerifier(self)
         filename, headers = opener.retrieve(url, test_filename, verif.callback)
-        try:
-            self.assertEqual(filename, test_filename)
-            check_retrieve(opener, filename, headers)
-            self.assert_(os.path.isfile(filename))
-        finally:
-            os.remove(filename)
+        self.assertEqual(filename, test_filename)
+        self._check_retrieve(url, filename, headers)
+        self.assert_(os.path.isfile(filename))
 
+    def test_retrieve(self):
+        # not passing an explicit filename downloads to a temporary file
+        # using a Request object instead of a URL works
+        url = urljoin(self.uri, "/mechanize/")
         opener = mechanize.build_opener()
         verif = CallbackVerifier(self)
-        filename, headers = opener.retrieve(url, reporthook=verif.callback)
-        check_retrieve(opener, filename, headers)
+        request = mechanize.Request(url)
+        filename, headers = opener.retrieve(request, reporthook=verif.callback)
+        self.assertEquals(request.visit, False)
+        self._check_retrieve(url, filename, headers)
+        opener.close()
         # closing the opener removed the temporary file
         self.failIf(os.path.isfile(filename))
 
+    def test_urlretrieve(self):
+        timeout_log = self._monkey_patch_socket()
+        timeout = 10.
+        url = urljoin(self.uri, "/mechanize/")
+        verif = CallbackVerifier(self)
+        filename, headers = mechanize.urlretrieve(url,
+                                                  reporthook=verif.callback,
+                                                  timeout=timeout)
+        timeout_log.stop()
+        self._check_retrieve(url, filename, headers)
+        timeout_log.verify(timeout)
+
     def test_reload_read_incomplete(self):
         from mechanize import Browser
         browser = Browser()
@@ -374,6 +534,76 @@ class FunctionalTests(TestCase):
 ##         r.close()
 ##         self.assert_(data1 != data2)
 
+
+class CookieJarTests(TestCase):
+
+    def test_mozilla_cookiejar(self):
+        filename = tempfile.mktemp()
+        try:
+            def get_cookiejar():
+                cj = mechanize.MozillaCookieJar(filename=filename)
+                try:
+                    cj.revert()
+                except IOError, exc:
+                    if exc.errno != errno.ENOENT:
+                        raise
+                return cj
+            def commit(cj):
+                cj.save()
+            self._test_cookiejar(get_cookiejar, commit)
+        finally:
+            try:
+                os.remove(filename)
+            except OSError, exc:
+                if exc.errno != errno.ENOENT:
+                    raise
+
+    def test_firefox3_cookiejar(self):
+        try:
+            mechanize.Firefox3CookieJar
+        except AttributeError:
+            # firefox 3 cookiejar is only supported in Python 2.5 and later;
+            # also, sqlite3 must be available
+            return
+
+        filename = tempfile.mktemp()
+        try:
+            def get_cookiejar():
+                hide_experimental_warnings()
+                try:
+                    cj = mechanize.Firefox3CookieJar(filename=filename)
+                finally:
+                    reset_experimental_warnings()
+                cj.connect()
+                return cj
+            def commit(cj):
+                pass
+            self._test_cookiejar(get_cookiejar, commit)
+        finally:
+            os.remove(filename)
+
+    def _test_cookiejar(self, get_cookiejar, commit):
+        cookiejar = get_cookiejar()
+        br = mechanize.Browser()
+        br.set_cookiejar(cookiejar)
+        br.set_handle_refresh(False)
+        url = urljoin(self.uri, "/cgi-bin/cookietest.cgi")
+        # no cookie was set on the first request
+        html = br.open(url).read()
+        self.assertEquals(html.find("Your browser supports cookies!"), -1)
+        self.assertEquals(len(cookiejar), 1)
+        # ... but now we have the cookie
+        html = br.open(url).read()
+        self.assert_("Your browser supports cookies!" in html)
+        commit(cookiejar)
+
+        # should still have the cookie when we load afresh
+        cookiejar = get_cookiejar()
+        br.set_cookiejar(cookiejar)
+        html = br.open(url).read()
+        self.assert_("Your browser supports cookies!" in html)
+
+
 class CallbackVerifier:
     # for .test_urlretrieve()
     def __init__(self, testcase):
@@ -387,6 +617,8 @@ class CallbackVerifier:
 if __name__ == "__main__":
     import sys
     sys.path.insert(0, "test-tools")
+    test_path = os.path.join(os.path.dirname(sys.argv[0]), "test")
+    sys.path.insert(0, test_path)
     import testprogram
     USAGE_EXAMPLES = """
 Examples:
@@ -401,9 +633,8 @@ Examples:
                  - start a local Twisted HTTP server and run the functional
                    tests against that, rather than against SourceForge
                    (quicker!)
-                   Note not all the functional tests use the local server yet
-                   -- some currently always access the internet regardless of
-                   this option and the --uri option.
+                   If this option doesn't work on Windows/Mac, somebody please
+                   tell me about it, or I'll never find out...
 """
     prog = testprogram.TestProgram(
         ["functional_tests"],
