@@ -17,8 +17,6 @@ __all__ = [
     'FileHandler',
     'FormNotFoundError',
     'FormsFactory',
-    'GopherError',
-    'GopherHandler',
     'HTTPBasicAuthHandler',
     'HTTPCookieProcessor',
     'HTTPDefaultErrorHandler',
@@ -82,6 +80,9 @@ __all__ = [
     'urlopen',
     'urlretrieve']
 
+import logging
+import sys
+
 from _mechanize import __version__
 
 # high-level stateful browser-style interface
@@ -115,11 +116,25 @@ except ImportError:
 
 # cookies
 from _clientcookie import Cookie, CookiePolicy, DefaultCookiePolicy, \
-     CookieJar, FileCookieJar, LoadError, request_host
+     CookieJar, FileCookieJar, LoadError, request_host_lc as request_host, \
+     effective_request_host
 from _lwpcookiejar import LWPCookieJar, lwp_cookie_str
+# 2.4 raises SyntaxError due to generator / try/finally use
+if sys.version_info[:2] > (2,4):
+    try:
+        import sqlite3
+    except ImportError:
+        pass
+    else:
+        from _firefox3cookiejar import Firefox3CookieJar
 from _mozillacookiejar import MozillaCookieJar
 from _msiecookiejar import MSIECookieJar
 
 # If you hate the idea of turning bugs into warnings, do:
 # import mechanize; mechanize.USE_BARE_EXCEPT = False
 USE_BARE_EXCEPT = True
+
+logger = logging.getLogger("mechanize")
+if logger.level is logging.NOTSET:
+    logger.setLevel(logging.CRITICAL)
+del logger
