@@ -9,21 +9,31 @@ included with the distribution).
 
 """
 
-import urllib2, sys, copy, re
+import urllib2, copy, re, os, urllib
+
 
-from _useragent import UserAgentBase
 from _html import DefaultFactory
 import _response
 import _request
 import _rfc3986
+import _sockettimeout
+from _useragent import UserAgentBase
 
-__version__ = (0, 1, 7, "b", None)  # 0.1.7b
+__version__ = (0, 1, 11, None, None)  # 0.1.11
 
 class BrowserStateError(Exception): pass
 class LinkNotFoundError(Exception): pass
 class FormNotFoundError(Exception): pass
 
 
+def sanepathname2url(path):
+    urlpath = urllib.pathname2url(path)
+    if os.name == "nt" and urlpath.startswith("///"):
+        urlpath = urlpath[2:]
+    # XXX don't ask me about the mac...
+    return urlpath
+
+
 class History:
     """
 
@@ -154,13 +164,7 @@ class Browser(UserAgentBase):
         self.follow_link = self.find_link = None
 
     def set_handle_referer(self, handle):
-        """Set whether to add Referer header to each request.
-
-        This base class does not implement this feature (so don't turn this on
-        if you're using this base class directly), but the subclass
-        mechanize.Browser does.
-
-        """
+        """Set whether to add Referer header to each request."""
         self._set_handler("_referer", handle)
         self._handle_referer = bool(handle)
 
@@ -184,11 +188,12 @@ class Browser(UserAgentBase):
             request.add_unredirected_header("Referer", referer)
         return request
 
-    def open_novisit(self, url, data=None):
+    def open_novisit(self, url, data=None,
+                     timeout=_sockettimeout._GLOBAL_DEFAULT_TIMEOUT):
         """Open a URL without visiting it.
 
-        The browser state (including .request, .response(), history, forms and
-        links) are all left unchanged by calling this function.
+        Browser state (including request, response, history, forms and links)
+        is left unchanged by calling this function.
 
         The interface is the same as for .open().
 
@@ -197,12 +202,14 @@ class Browser(UserAgentBase):
         See also .retrieve().
 
         """
-        return self._mech_open(url, data, visit=False)
+        return self._mech_open(url, data, visit=False, timeout=timeout)
 
-    def open(self, url, data=None):
-        return self._mech_open(url, data)
+    def open(self, url, data=None,
+             timeout=_sockettimeout._GLOBAL_DEFAULT_TIMEOUT):
+        return self._mech_open(url, data, timeout=timeout)
 
-    def _mech_open(self, url, data=None, update_history=True, visit=None):
+    def _mech_open(self, url, data=None, update_history=True, visit=None,
+                   timeout=_sockettimeout._GLOBAL_DEFAULT_TIMEOUT):
         try:
             url.get_full_url
         except AttributeError:
@@ -216,7 +223,7 @@ class Browser(UserAgentBase):
                         "not viewing any document")
                 url = _rfc3986.urljoin(self._response.geturl(), url)
 
-        request = self._request(url, data, visit)
+        request = self._request(url, data, visit, timeout)
         visit = request.visit
         if visit is None:
             visit = True
@@ -275,6 +282,11 @@ class Browser(UserAgentBase):
         """
         return copy.copy(self._response)
 
+    def open_local_file(self, filename):
+        path = sanepathname2url(os.path.abspath(filename))
+        url = 'file://'+path
+        return self.open(url)
+
     def set_response(self, response):
         """Replace current response with (a copy of) response.
 
@@ -365,7 +377,7 @@ class Browser(UserAgentBase):
 
         The cookie is added in the same way as if it had arrived with the
         current response, as a result of the current request.  This means that,
-        for example, it is not appropriate to set the cookie based on the
+        for example, if it is not appropriate to set the cookie based on the
         current request, no cookie will be set.
 
         The cookie will be returned automatically with subsequent responses
@@ -417,8 +429,8 @@ class Browser(UserAgentBase):
         """Return the global form object, or None if the factory implementation
         did not supply one.
 
-        The "global" form object contains all controls that are not descendants of
-        any FORM element.
+        The "global" form object contains all controls that are not descendants
+        of any FORM element.
 
         The returned form object implements the ClientForm.HTMLForm interface.
 
@@ -438,16 +450,15 @@ class Browser(UserAgentBase):
         return self._factory.is_html
 
     def encoding(self):
-        """"""
         if self._response is None:
             raise BrowserStateError("not viewing any document")
         return self._factory.encoding
 
     def title(self):
-        """Return title, or None if there is no title element in the document.
+        r"""Return title, or None if there is no title element in the document.
 
-        Tags are stripped or textified as described in docs for
-        PullParser.get_text() method of pullparser module.
+        Treatment of any tag children of attempts to follow Firefox and IE
+        (currently, tags are preserved).
 
         """
         if not self.viewing_html():
@@ -479,7 +490,10 @@ class Browser(UserAgentBase):
         nr, if supplied, is the sequence number of the form (where 0 is the
         first).  Note that control 0 is the first form matching all the other
         arguments (if supplied); it is not necessarily the first control in the
-        form.
+        form.  The "global form" (consisting of all form controls not contained
+        in any FORM element) is considered not to be part of this sequence and
+        to have no name, so will not be matched unless both name and nr are
+        None.
 
         """
         if not self.viewing_html():
@@ -488,6 +502,12 @@ class Browser(UserAgentBase):
             raise ValueError(
                 "at least one argument must be supplied to specify form")
 
+        global_form = self._factory.global_form
+        if nr is None and name is None and \
+               predicate is not None and predicate(global_form):
+            self.form = global_form
+            return
+
         orig_nr = nr
         for form in self.forms():
             if name is not None and name != form.name:
