@@ -11,13 +11,14 @@ included with the distribution).
 
 """
 
-import sys, warnings, urllib2
+import warnings
 
-import _opener
-import _urllib2
 import _auth
 import _gzip
+import _opener
 import _response
+import _sockettimeout
+import _urllib2
 
 
 class UserAgentBase(_opener.OpenerDirector):
@@ -50,7 +51,6 @@ class UserAgentBase(_opener.OpenerDirector):
         # CacheFTPHandler is buggy, at least in 2.3, so we don't use it
         "ftp": _urllib2.FTPHandler,
         "file": _urllib2.FileHandler,
-        "gopher": _urllib2.GopherHandler,
 
         # other handlers
         "_unknown": _urllib2.UnknownHandler,
@@ -77,7 +77,7 @@ class UserAgentBase(_opener.OpenerDirector):
         "_debug_response_body": _urllib2.HTTPResponseDebugProcessor,
         }
 
-    default_schemes = ["http", "ftp", "file", "gopher"]
+    default_schemes = ["http", "ftp", "file"]
     default_others = ["_unknown", "_http_error", "_http_request_upgrade",
                       "_http_default_error",
                       ]
@@ -307,7 +307,8 @@ class UserAgentBase(_opener.OpenerDirector):
             if obj is not None:
                 newhandler = handler_class(obj)
             else:
-                newhandler = handler_class(*constructor_args, **constructor_kwds)
+                newhandler = handler_class(
+                    *constructor_args, **constructor_kwds)
         else:
             newhandler = None
         self._replace_handler(name, newhandler)
@@ -337,12 +338,15 @@ class UserAgent(UserAgentBase):
         """Make response objects .seek()able."""
         self._seekable = bool(handle)
 
-    def open(self, fullurl, data=None):
+    def open(self, fullurl, data=None,
+             timeout=_sockettimeout._GLOBAL_DEFAULT_TIMEOUT):
         if self._seekable:
-            def bound_open(fullurl, data=None):
-                return UserAgentBase.open(self, fullurl, data)
+            def bound_open(fullurl, data=None,
+                           timeout=_sockettimeout._GLOBAL_DEFAULT_TIMEOUT):
+                return UserAgentBase.open(self, fullurl, data, timeout)
             response = _opener.wrapped_open(
-                bound_open, _response.seek_wrapped_response, fullurl, data)
+                bound_open, _response.seek_wrapped_response, fullurl, data,
+                timeout)
         else:
             response = UserAgentBase.open(self, fullurl, data)
         return response
