@@ -13,17 +13,11 @@ python test.py --help
 MODULE_NAMES = ["test_date", "test_browser", "test_response", "test_cookies",
                 "test_headers", "test_urllib2", "test_pullparser",
                 "test_useragent", "test_html", "test_opener",
+#                "test_performance",  # too slow, run from release script
                 ]
 
 import sys, os, logging, glob
 
-#level = logging.DEBUG
-#level = logging.INFO
-#level = logging.WARNING
-#level = logging.NOTSET
-#logging.getLogger("mechanize").setLevel(level)
-#logging.getLogger("mechanize").addHandler(logging.StreamHandler(sys.stdout))
-
 
 if __name__ == "__main__":
     # XXX
@@ -45,6 +39,18 @@ if __name__ == "__main__":
     run_unittests = "-u" not in sys.argv
     if not run_unittests:
         sys.argv.remove("-u")
+    log = "-l" in sys.argv
+    if log:
+        sys.argv.remove("-l")
+        level = logging.DEBUG
+#         level = logging.INFO
+#         level = logging.WARNING
+#         level = logging.NOTSET
+        logger = logging.getLogger("mechanize")
+        logger.setLevel(level)
+        handler = logging.StreamHandler(sys.stdout)
+        handler.setLevel(level)
+        logger.addHandler(handler)
 
     # import local copy of Python 2.5 doctest
     assert os.path.isdir("test")
@@ -59,8 +65,8 @@ if __name__ == "__main__":
     import doctest
     import testprogram
 
-    import coverage
     if run_coverage:
+        import coverage
         print 'running coverage'
         coverage.erase()
         coverage.start()
@@ -76,39 +82,36 @@ if __name__ == "__main__":
         # run .doctest files needing special support
         common_globs = {"mechanize": mechanize}
         pm_doctest_filename = os.path.join(
-            "test", "test_password_manager.doctest")
+            "test", "test_password_manager.special_doctest")
         for globs in [
             {"mgr_class": mechanize.HTTPPasswordMgr},
             {"mgr_class": mechanize.HTTPProxyPasswordMgr},
             ]:
             globs.update(common_globs)
-            doctest.testfile(
-                pm_doctest_filename,
-                #os.path.join("test", "test_scratch.doctest"),
-                globs=globs,
-                )
+            doctest.testfile(pm_doctest_filename, globs=globs)
+        try:
+            import robotparser
+        except ImportError:
+            pass
+        else:
+            doctest.testfile(os.path.join(
+                    "test", "test_robotfileparser.special_doctest"))
 
         # run .doctest files
-        special_doctests = [pm_doctest_filename,
-                            os.path.join("test", "test_scratch.doctest"),
-                            ]
         doctest_files = glob.glob(os.path.join("test", "*.doctest"))
-
-        for dt in special_doctests:
-            if dt in doctest_files:
-                doctest_files.remove(dt)
         for df in doctest_files:
             doctest.testfile(df)
 
         # run doctests in docstrings
         from mechanize import _headersutil, _auth, _clientcookie, _pullparser, \
-             _http, _rfc3986
+             _http, _rfc3986, _useragent
         doctest.testmod(_headersutil)
         doctest.testmod(_rfc3986)
         doctest.testmod(_auth)
         doctest.testmod(_clientcookie)
         doctest.testmod(_pullparser)
         doctest.testmod(_http)
+        doctest.testmod(_useragent)
 
     if run_unittests:
         # run vanilla unittest tests
