@@ -1,9 +1,13 @@
 """Tests for _ClientCookie."""
 
-import urllib2, re, os, StringIO, mimetools, time
+import sys, urllib2, re, os, StringIO, mimetools, time, tempfile, errno, inspect
 from time import localtime
 from unittest import TestCase
 
+from mechanize._util import hide_experimental_warnings, \
+    reset_experimental_warnings
+
+
 class FakeResponse:
     def __init__(self, headers=[], url=None):
         """
@@ -35,6 +39,151 @@ def _interact(cookiejar, url, set_cookie_hdrs, hdr_name):
     return cookie_hdr
 
 
+class TempfileTestMixin:
+
+    def setUp(self):
+        self._tempfiles = []
+
+    def tearDown(self):
+        for fn in self._tempfiles:
+            try:
+                os.remove(fn)
+            except IOError, exc:
+                if exc.errno != errno.ENOENT:
+                    raise
+
+    def mktemp(self):
+        fn = tempfile.mktemp()
+        self._tempfiles.append(fn)
+        return fn
+
+
+def caller():
+    return sys._getframe().f_back.f_back.f_code.co_name
+
+def attribute_names(obj):
+    return set([spec[0] for spec in inspect.getmembers(obj)
+                if not spec[0].startswith("__")])
+
+class CookieJarInterfaceTests(TestCase):
+
+    def test_add_cookie_header(self):
+        from mechanize import CookieJar
+        # verify only these methods are used
+        class MockRequest(object):
+            def __init__(self):
+                self.added_headers = []
+                self.called = set()
+            def log_called(self):
+                self.called.add(caller())
+            def get_full_url(self):
+                self.log_called()
+                return "https://example.com:443"
+            def get_host(self):
+                self.log_called()
+                return "example.com:443"
+            def get_type(self):
+                self.log_called()
+                return "https"
+            def has_header(self, header_name):
+                self.log_called()
+                return False
+            def get_header(self, header_name, default=None):
+                self.log_called()
+                pass  # currently not called
+            def header_items(self):
+                self.log_called()
+                pass  # currently not called
+            def add_unredirected_header(self, key, val):
+                self.log_called()
+                self.added_headers.append((key, val))
+            def is_unverifiable(self):
+                self.log_called()
+                return False
+            @property
+            def port(self):
+                import traceback; traceback.print_stack()
+                self.log_called()
+                pass # currently not used, since urllib2 always sets .port None
+        jar = CookieJar()
+        interact_netscape(jar, "https://example.com:443",
+                          "foo=bar; port=443; secure")
+        request = MockRequest()
+        jar.add_cookie_header(request)
+        expect_called = attribute_names(MockRequest) - set(
+            ["port", "get_header", "header_items", "log_called"])
+        self.assertEquals(request.called, expect_called)
+        self.assertEquals(request.added_headers, [("Cookie", "foo=bar")])
+
+    def test_extract_cookies(self):
+        from mechanize import CookieJar
+
+        # verify only these methods are used
+
+        class StubMessage(object):
+            def getheaders(self, name):
+                return ["foo=bar; port=443"]
+
+        class StubResponse(object):
+            def info(self):
+                return StubMessage()
+
+        class StubRequest(object):
+            def __init__(self):
+                self.added_headers = []
+                self.called = set()
+            def log_called(self):
+                self.called.add(caller())
+            def get_full_url(self):
+                self.log_called()
+                return "https://example.com:443"
+            def get_host(self):
+                self.log_called()
+                return "example.com:443"
+            def is_unverifiable(self):
+                self.log_called()
+                return False
+            @property
+            def port(self):
+                import traceback; traceback.print_stack()
+                self.log_called()
+                pass # currently not used, since urllib2 always sets .port None
+        jar = CookieJar()
+        response = StubResponse()
+        request = StubRequest()
+        jar.extract_cookies(response, request)
+        expect_called = attribute_names(StubRequest) - set(
+            ["port", "log_called"])
+        self.assertEquals(request.called, expect_called)
+        self.assertEquals([(cookie.name, cookie.value) for cookie in jar],
+                          [("foo", "bar")])
+
+    def test_unverifiable(self):
+        from mechanize._clientcookie import request_is_unverifiable
+        # .unverifiable was added in mechanize, .is_unverifiable() later got
+        # added in cookielib.  XXX deprecate .unverifiable
+        class StubRequest(object):
+            def __init__(self, attrs):
+                self._attrs = attrs
+                self.accessed = set()
+            def __getattr__(self, name):
+                self.accessed.add(name)
+                try:
+                    return self._attrs[name]
+                except KeyError:
+                    raise AttributeError(name)
+
+        request = StubRequest(dict(is_unverifiable=lambda: False))
+        self.assertEquals(request_is_unverifiable(request), False)
+
+        request = StubRequest(dict(is_unverifiable=lambda: False,
+                                   unverifiable=True))
+        self.assertEquals(request_is_unverifiable(request), False)
+
+        request = StubRequest(dict(unverifiable=False))
+        self.assertEquals(request_is_unverifiable(request), False)
+
+
 class CookieTests(TestCase):
     # XXX
     # Get rid of string comparisons where not actually testing str / repr.
@@ -82,6 +231,37 @@ class CookieTests(TestCase):
 ##   just the 7 special TLD's listed in their spec. And folks rely on
 ##   that...
 
+    def test_policy(self):
+        import mechanize
+        policy = mechanize.DefaultCookiePolicy()
+        jar = mechanize.CookieJar()
+        jar.set_policy(policy)
+        self.assertEquals(jar.get_policy(), policy)
+
+    def test_make_cookies_doesnt_change_jar_state(self):
+        from mechanize import CookieJar, Request, Cookie
+        from mechanize._util import time2netscape
+        from mechanize._response import test_response
+        cookie = Cookie(0, "spam", "eggs",
+                        "80", False,
+                        "example.com", False, False,
+                        "/", False,
+                        False,
+                        None,
+                        False,
+                        "",
+                        "",
+                        {})
+        jar = CookieJar()
+        jar._policy._now = jar._now = int(time.time())
+        jar.set_cookie(cookie)
+        self.assertEquals(len(jar), 1)
+        set_cookie = "spam=eggs; expires=%s" % time2netscape(time.time()- 1000)
+        url = "http://example.com/"
+        response = test_response(url=url, headers=[("Set-Cookie", set_cookie)])
+        jar.make_cookies(response, Request(url))
+        self.assertEquals(len(jar), 1)
+
     def test_domain_return_ok(self):
         # test optimization: .domain_return_ok() should filter out most
         # domains in the CookieJar before we try to access them (because that
@@ -118,7 +298,7 @@ class CookieTests(TestCase):
 
         # missing = sign in Cookie: header is regarded by Mozilla as a missing
         # NAME.  WE regard it as a missing VALUE.
-        filename = os.path.abspath("cookies2.txt")
+        filename = tempfile.mktemp()
         c = MozillaCookieJar(filename)
         interact_netscape(c, "http://www.acme.com/", 'eggs')
         interact_netscape(c, "http://www.acme.com/", '"spam"; path=/foo/')
@@ -347,28 +527,40 @@ class CookieTests(TestCase):
                       headers={"Host": "www.acme.com:4321"})
         assert request_port(req) == DEFAULT_HTTP_PORT
 
-    def test_request_host(self):
+    def test_request_host_lc(self):
         from mechanize import Request
-        from mechanize._clientcookie import request_host
+        from mechanize._clientcookie import request_host_lc
         # this request is illegal (RFC2616, 14.2.3)
         req = Request("http://1.1.1.1/",
                       headers={"Host": "www.acme.com:80"})
         # libwww-perl wants this response, but that seems wrong (RFC 2616,
         # section 5.2, point 1., and RFC 2965 section 1, paragraph 3)
-        #assert request_host(req) == "www.acme.com"
-        assert request_host(req) == "1.1.1.1"
+        #assert request_host_lc(req) == "www.acme.com"
+        assert request_host_lc(req) == "1.1.1.1"
         req = Request("http://www.acme.com/",
                       headers={"Host": "irrelevant.com"})
-        assert request_host(req) == "www.acme.com"
+        assert request_host_lc(req) == "www.acme.com"
         # not actually sure this one is valid Request object, so maybe should
-        # remove test for no host in url in request_host function?
+        # remove test for no host in url in request_host_lc function?
         req = Request("/resource.html",
                       headers={"Host": "www.acme.com"})
-        assert request_host(req) == "www.acme.com"
+        assert request_host_lc(req) == "www.acme.com"
         # port shouldn't be in request-host
         req = Request("http://www.acme.com:2345/resource.html",
                       headers={"Host": "www.acme.com:5432"})
-        assert request_host(req) == "www.acme.com"
+        assert request_host_lc(req) == "www.acme.com"
+        # the _lc function lower-cases the result
+        req = Request("http://EXAMPLE.com")
+        assert request_host_lc(req) == "example.com"
+
+    def test_effective_request_host(self):
+        from mechanize import Request, effective_request_host
+        self.assertEquals(
+            effective_request_host(Request("http://www.EXAMPLE.com/spam")),
+            "www.EXAMPLE.com")
+        self.assertEquals(
+            effective_request_host(Request("http://bob/spam")),
+            "bob.local")
 
     def test_is_HDN(self):
         from mechanize._clientcookie import is_HDN
@@ -623,7 +815,7 @@ class CookieTests(TestCase):
         cookies = c.make_cookies(res, req)
         c.set_cookie(cookies[0])
         assert len(c) == 2
-        # ... and check is doesn't get returned
+        # ... and check it doesn't get returned
         c.add_cookie_header(req)
         assert not req.has_header("Cookie")
 
@@ -852,6 +1044,8 @@ class CookieTests(TestCase):
             ["Set-Cookie2: a=foo; path=/; Version=1; domain"],
             # bad max-age
             ["Set-Cookie: b=foo; max-age=oops"],
+            # bad version
+            ["Set-Cookie: b=foo; version=spam"],
             ]:
             c = cookiejar_from_cookie_headers(headers)
             # these bad cookies shouldn't be set
@@ -863,8 +1057,227 @@ class CookieTests(TestCase):
         cookie = c._cookies["www.example.com"]["/"]["c"]
         assert cookie.expires is None
 
+    def test_cookies_for_request(self):
+        from mechanize import CookieJar, Request
 
-class LWPCookieTests(TestCase):
+        cj = CookieJar()
+        interact_netscape(cj, "http://example.com/", "short=path")
+        interact_netscape(cj, "http://example.com/longer/path", "longer=path")
+        for_short_path = cj.cookies_for_request(Request("http://example.com/"))
+        self.assertEquals([cookie.name for cookie in for_short_path],
+                          ["short"])
+        for_long_path = cj.cookies_for_request(Request(
+                "http://example.com/longer/path"))
+        self.assertEquals([cookie.name for cookie in for_long_path],
+                          ["longer", "short"])
+
+
+class CookieJarPersistenceTests(TempfileTestMixin, TestCase):
+
+    def _interact(self, cj):
+        year_plus_one = localtime(time.time())[0] + 1
+        interact_2965(cj, "http://www.acme.com/",
+                      "foo1=bar; max-age=100; Version=1")
+        interact_2965(cj, "http://www.acme.com/",
+                      'foo2=bar; port="80"; max-age=100; Discard; Version=1')
+        interact_2965(cj, "http://www.acme.com/", "foo3=bar; secure; Version=1")
+
+        expires = "expires=09-Nov-%d 23:12:40 GMT" % (year_plus_one,)
+        interact_netscape(cj, "http://www.foo.com/",
+                          "fooa=bar; %s" % expires)
+        interact_netscape(cj, "http://www.foo.com/",
+                          "foob=bar; Domain=.foo.com; %s" % expires)
+        interact_netscape(cj, "http://www.foo.com/",
+                          "fooc=bar; Domain=www.foo.com; %s" % expires)
+
+    def test_firefox3_cookiejar_restore(self):
+        try:
+            from mechanize import Firefox3CookieJar
+        except ImportError:
+            pass
+        else:
+            from mechanize import DefaultCookiePolicy
+            filename = self.mktemp()
+            def create_cookiejar():
+                hide_experimental_warnings()
+                try:
+                    cj = Firefox3CookieJar(
+                        filename, policy=DefaultCookiePolicy(rfc2965=True))
+                finally:
+                    reset_experimental_warnings()
+                cj.connect()
+                return cj
+            cj = create_cookiejar()
+            self._interact(cj)
+            self.assertEquals(len(cj), 6)
+            cj.close()
+            cj = create_cookiejar()
+            self.assert_("name='foo1', value='bar'" in repr(cj))
+            self.assertEquals(len(cj), 4)
+
+    def test_firefox3_cookiejar_iteration(self):
+        try:
+            from mechanize import Firefox3CookieJar
+        except ImportError:
+            pass
+        else:
+            from mechanize import DefaultCookiePolicy, Cookie
+            filename = self.mktemp()
+            hide_experimental_warnings()
+            try:
+                cj = Firefox3CookieJar(
+                    filename, policy=DefaultCookiePolicy(rfc2965=True))
+            finally:
+                reset_experimental_warnings()
+            cj.connect()
+            self._interact(cj)
+            summary = "\n".join([str(cookie) for cookie in cj])
+            self.assertEquals(summary,
+                              """\
+<Cookie foo2=bar for www.acme.com:80/>
+<Cookie foo3=bar for www.acme.com/>
+<Cookie foo1=bar for www.acme.com/>
+<Cookie fooa=bar for www.foo.com/>
+<Cookie foob=bar for .foo.com/>
+<Cookie fooc=bar for .www.foo.com/>""")
+
+    def test_firefox3_cookiejar_clear(self):
+        try:
+            from mechanize import Firefox3CookieJar
+        except ImportError:
+            pass
+        else:
+            from mechanize import DefaultCookiePolicy, Cookie
+            filename = self.mktemp()
+            hide_experimental_warnings()
+            try:
+                cj = Firefox3CookieJar(
+                    filename, policy=DefaultCookiePolicy(rfc2965=True))
+            finally:
+                reset_experimental_warnings()
+            cj.connect()
+            self._interact(cj)
+            cj.clear("www.acme.com", "/", "foo2")
+            def summary(): return "\n".join([str(cookie) for cookie in cj])
+            self.assertEquals(summary(),
+                              """\
+<Cookie foo3=bar for www.acme.com/>
+<Cookie foo1=bar for www.acme.com/>
+<Cookie fooa=bar for www.foo.com/>
+<Cookie foob=bar for .foo.com/>
+<Cookie fooc=bar for .www.foo.com/>""")
+            cj.clear("www.acme.com")
+            self.assertEquals(summary(),
+                              """\
+<Cookie fooa=bar for www.foo.com/>
+<Cookie foob=bar for .foo.com/>
+<Cookie fooc=bar for .www.foo.com/>""")
+            # if name is given, so must path and domain
+            self.assertRaises(ValueError, cj.clear, domain=".foo.com",
+                              name="foob")
+            # nonexistent domain
+            self.assertRaises(KeyError, cj.clear, domain=".spam.com")
+
+    def test_firefox3_cookiejar_add_cookie_header(self):
+        try:
+            from mechanize import Firefox3CookieJar
+        except ImportError:
+            pass
+        else:
+            from mechanize import DefaultCookiePolicy, Request
+            filename = self.mktemp()
+            hide_experimental_warnings()
+            try:
+                cj = Firefox3CookieJar(filename)
+            finally:
+                reset_experimental_warnings()
+            cj.connect()
+            # Session cookies (true .discard) and persistent cookies (false
+            # .discard) are stored differently.  Check they both get sent.
+            year_plus_one = localtime(time.time())[0] + 1
+            expires = "expires=09-Nov-%d 23:12:40 GMT" % (year_plus_one,)
+            interact_netscape(cj, "http://www.foo.com/", "fooa=bar")
+            interact_netscape(cj, "http://www.foo.com/",
+                              "foob=bar; %s" % expires)
+            ca, cb = cj
+            self.assert_(ca.discard)
+            self.assertFalse(cb.discard)
+            request = Request("http://www.foo.com/")
+            cj.add_cookie_header(request)
+            self.assertEquals(request.get_header("Cookie"),
+                              "fooa=bar; foob=bar")
+
+    def test_mozilla_cookiejar(self):
+        # Save / load Mozilla/Netscape cookie file format.
+        from mechanize import MozillaCookieJar, DefaultCookiePolicy
+        filename = tempfile.mktemp()
+        c = MozillaCookieJar(filename,
+                             policy=DefaultCookiePolicy(rfc2965=True))
+        self._interact(c)
+
+        def save_and_restore(cj, ignore_discard, filename=filename):
+            from mechanize import MozillaCookieJar, DefaultCookiePolicy
+            try:
+                cj.save(ignore_discard=ignore_discard)
+                new_c = MozillaCookieJar(filename,
+                                         DefaultCookiePolicy(rfc2965=True))
+                new_c.load(ignore_discard=ignore_discard)
+            finally:
+                try: os.unlink(filename)
+                except OSError: pass
+            return new_c
+
+        new_c = save_and_restore(c, True)
+        assert len(new_c) == 6  # none discarded
+        assert repr(new_c).find("name='foo1', value='bar'") != -1
+
+        new_c = save_and_restore(c, False)
+        assert len(new_c) == 4  # 2 of them discarded on save
+        assert repr(new_c).find("name='foo1', value='bar'") != -1
+
+    def test_mozilla_cookiejar_embedded_tab(self):
+        from mechanize import MozillaCookieJar
+        filename = tempfile.mktemp()
+        fh = open(filename, "w")
+        try:
+            fh.write(
+                MozillaCookieJar.header + "\n" +
+                "a.com\tFALSE\t/\tFALSE\t\tname\tval\tstillthevalue\n"
+                "a.com\tFALSE\t/\tFALSE\t\tname2\tvalue\n")
+            fh.close()
+            cj = MozillaCookieJar(filename)
+            cj.revert(ignore_discard=True)
+            cookies = cj._cookies["a.com"]["/"]
+            self.assertEquals(cookies["name"].value, "val\tstillthevalue")
+            self.assertEquals(cookies["name2"].value, "value")
+        finally:
+            try:
+                os.remove(filename)
+            except IOError, exc:
+                if exc.errno != errno.ENOENT:
+                    raise
+
+    def test_mozilla_cookiejar_initial_dot_violation(self):
+        from mechanize import MozillaCookieJar, LoadError
+        filename = tempfile.mktemp()
+        fh = open(filename, "w")
+        try:
+            fh.write(
+                MozillaCookieJar.header + "\n" +
+                ".a.com\tFALSE\t/\tFALSE\t\tname\tvalue\n")
+            fh.close()
+            cj = MozillaCookieJar(filename)
+            self.assertRaises(LoadError, cj.revert, ignore_discard=True)
+        finally:
+            try:
+                os.remove(filename)
+            except IOError, exc:
+                if exc.errno != errno.ENOENT:
+                    raise
+
+
+
+class LWPCookieTests(TestCase, TempfileTestMixin):
     # Tests taken from libwww-perl, with a few modifications.
 
     def test_netscape_example_1(self):
@@ -1262,7 +1675,7 @@ class LWPCookieTests(TestCase):
         assert len(c) == 6
 
         # save and restore
-        filename = "lwp-cookies.txt"
+        filename = tempfile.mktemp()
 
         try:
             c.save(filename, ignore_discard=True)
@@ -1300,50 +1713,6 @@ class LWPCookieTests(TestCase):
         # unicode URL doesn't raise exception, as it used to!
         cookie = interact_2965(c, u"http://www.acme.com/\xfc")
 
-    def test_mozilla(self):
-        # Save / load Mozilla/Netscape cookie file format.
-        from mechanize import MozillaCookieJar, DefaultCookiePolicy
-
-        year_plus_one = localtime(time.time())[0] + 1
-
-        filename = "cookies.txt"
-
-        c = MozillaCookieJar(filename,
-                             policy=DefaultCookiePolicy(rfc2965=True))
-        interact_2965(c, "http://www.acme.com/",
-                      "foo1=bar; max-age=100; Version=1")
-        interact_2965(c, "http://www.acme.com/",
-                      'foo2=bar; port="80"; max-age=100; Discard; Version=1')
-        interact_2965(c, "http://www.acme.com/", "foo3=bar; secure; Version=1")
-
-        expires = "expires=09-Nov-%d 23:12:40 GMT" % (year_plus_one,)
-        interact_netscape(c, "http://www.foo.com/",
-                          "fooa=bar; %s" % expires)
-        interact_netscape(c, "http://www.foo.com/",
-                          "foob=bar; Domain=.foo.com; %s" % expires)
-        interact_netscape(c, "http://www.foo.com/",
-                          "fooc=bar; Domain=www.foo.com; %s" % expires)
-
-        def save_and_restore(cj, ignore_discard, filename=filename):
-            from mechanize import MozillaCookieJar, DefaultCookiePolicy
-            try:
-                cj.save(ignore_discard=ignore_discard)
-                new_c = MozillaCookieJar(filename,
-                                         DefaultCookiePolicy(rfc2965=True))
-                new_c.load(ignore_discard=ignore_discard)
-            finally:
-                try: os.unlink(filename)
-                except OSError: pass
-            return new_c
-
-        new_c = save_and_restore(c, True)
-        assert len(new_c) == 6  # none discarded
-        assert repr(new_c).find("name='foo1', value='bar'") != -1
-
-        new_c = save_and_restore(c, False)
-        assert len(new_c) == 4  # 2 of them discarded on save
-        assert repr(new_c).find("name='foo1', value='bar'") != -1
-
     def test_netscape_misc(self):
         # Some additional Netscape cookies tests.
         from mechanize import CookieJar, Request
